#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <getopt.h>
#include <errno.h>
#include <inttypes.h>

#define RING_BUFFER_SIZE (8 << 10)

static struct option long_options[] = {
	{"mask", 1, 0, 'm'},
	{"test", 0, 0, 't'},
	{"help", 0, 0, 'h'},
	{0, 0, 0, 0}
};

const char *usage =
"early-print --mask <mask> --help\n"
"Usage options:\n"
"-h, --help             : Print this help\n";

void print_usage(int exit_code)
{
	printf(usage);
	exit(exit_code);
}

static void *map_early_print_memory(int test)
{
	int fd = -1;
	void *mmap_vaddr = 0;
	size_t size = (1 << 20);
	off_t pbase = (511 << 20);

        fd = open("/dev/mem", O_RDWR);
	if (fd < 0) {
                printf("Cannot open /dev/nlm_vuart: \"%s\"\n", strerror(errno));
                exit(-1);
        }

	mmap_vaddr = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, pbase);
	if (mmap_vaddr == MAP_FAILED) {
                printf("mmap error (%s)\n", strerror(errno));
		exit(-1);
        }
	printf("mmapped  virt %p size 0x%x pbase=%lx\n", mmap_vaddr, size, pbase);

	if (test){
		volatile unsigned int *ptr = mmap_vaddr;
		ptr[0] = 0xdeadbeef;
		printf("mmap[0] = %x, mmap[1] = %x\n", ptr[0], ptr[1]);
		exit(0);
	}

	return mmap_vaddr;
}

static void print_cpu(int cpu, char *buf)
{
	int i = 0;

	printf("================cpu(%d), buf=%p =================\n", cpu, buf);
	for (i = 0; i < RING_BUFFER_SIZE; i++) {
		printf("%c", buf[i]);
	}
}

int main(int argc, char *argv[])
{
	int test_mmap = 0;
	void *ep_base = 0;
	char *buf = 0;
	int i, opt_idx;
	int c;
	uint32_t mask = 0x01;

	while((c = getopt_long(argc, argv, "ahtm:", long_options, &opt_idx)) >= 0) {
		switch(c) {
		case 't':
			test_mmap = 1;
			break;
		case 'm':
			mask = (int)strtoul(optarg, 0, 16);
			break;
		case 'h':
			print_usage(0);
			break;
		default:
			break;
		}
	}

	printf("test_mmap = %d, mask = 0x%08x\n", test_mmap, mask);

	ep_base = (void *)map_early_print_memory(test_mmap);

	for (i = 0; i < 32; i++) {

		if ( (mask & (1 << i)) == 0) continue;

		buf = ep_base + i * RING_BUFFER_SIZE;
		print_cpu(i, buf);
	}
}
