#ifndef _QUEUE_H
#define _QUEUE_H

#define MAX_ELEMENTS 16384

#define QFULL  1
#define QEMPTY 2

#define barrier() __asm__ __volatile__("sync": : :"memory")

typedef volatile struct queue
{
	int head;
	int tail;
	int max_elements;
	int init_done;
	char data[MAX_ELEMENTS];
} queue_t;

static inline void init_queue(queue_t *q, int max_elements)
{
	q->head = 0;
	q->tail = 0;
	q->max_elements = max_elements;
	q->init_done = 1;
}

static inline int enqueue(queue_t *q, int element)
{
	barrier();
	if (((q->tail + 1) % q->max_elements) == q->head)
		return -QFULL;
	barrier();

	q->data[q->tail] = element;
	barrier();
	q->tail = (q->tail + 1) % q->max_elements;
	barrier();

	return 0;
}

static inline int dequeue(queue_t *q, int *element)
{
	barrier();
	if (q->head == q->tail)
		return -QEMPTY;

	barrier();
	*element = q->data[q->head];
	/* Write a dummy character for debugging purposes */
	//q->data[q->head] = '@';
	barrier();
	q->head = (q->head + 1) % q->max_elements;
	barrier();

	return 0;
}

#endif
