#include <stdio.h>
#ifndef __LINUX__
#include "mips-exts.h"
#endif

#include "queue.h"
#include "vuart.h"

volatile queue_t *vuart_queues;

/*
 * should be called by linux
 */
void uart_init(void *vuart_baseaddr, int nr_vuart, int initq)
{
	int i;

	vuart_queues = (queue_t *)vuart_baseaddr;

	if (initq) {
		for (i = 0; i < nr_vuart; i += 2 ) {
			/* 2 = 1 producer Q + 1 consumer Q */
			init_queue(&vuart_queues[i], MAX_ELEMENTS);
			init_queue(&vuart_queues[i+1], MAX_ELEMENTS);
		}
	}
}

char inbyte(int vuart, int qid)
{
	int c;
	queue_t *queue = &vuart_queues[(vuart * 2) + qid];

	if (queue->init_done != 1)
		return EOF;

	while (dequeue(queue, &c) != 0)
		;

	return c;
}

int inbyte_nowait(int vuart, int qid)
{
	int c;
	queue_t *queue = &vuart_queues[(vuart * 2) + qid];

	if (queue->init_done != 1)
		return EOF;

	if (dequeue(queue, &c) != 0)
		return EOF;

	return c;
}

void outbyte(char c, int vuart, int qid)
{
	queue_t *queue = &vuart_queues[(vuart * 2) + qid];

	if (queue->init_done != 1)
		return;

	while (enqueue(queue, c) != 0)
		;

	return;
}
